#include "main.hpp"

int Drehregler;  // Ausgabewert des Drehreglers
int speed;  // Das Wort "Geschwindigkeit" steht als Variable für den Ansteuerungswert am ESC.
int last_speed;

time_t run_time = 0;
time_t start_time = 0;
hw_timer_t * timer = NULL;

volatile double integ = 0.0;
volatile double derivative = 0.0;
volatile double esc_output = 0.0;
volatile double last_error = 0.0;
int esc_value = 0;
volatile double goal_speed = 1000; //1.0 * speed;
volatile double current_speed = 0;
volatile double err = 0;
volatile double output = 0.0;
const double kp = 0.01;
const double ki = 0.001;
const double kd = 0.0005;

void ISR_HALL1();
void ISR_HALL2();
void ISR_HALL3();
void ISR_HALL4();
void ISR_HALL5();
void ISR_HALL6();
void ISR_HALL7();
void ISR_HALL8();
void ISR_onTimer();

void data_generate();
void data_store();
void data_check();
void data_init();
void speed_set();
void count_secs(time_t* run_time);

#ifdef HAS_MQTT
void json_init();
#endif

void setup() 
{
  delay(1000);
  Serial.begin(115200);
  Serial.println("Init capture portal ...");

  randomSeed(micros());

  display.begin();
  capportal.begin();
  html_content = "<!DOCTYPE html><html><head><meta http-equiv=\"refresh\" content=\"0; url='/_ac'\" /></head><body></body></html>";  

  #ifdef HAS_MQTT
  mqtt.begin(mqttServer, mqttPort, mqttUser, mqttPassword);
  #endif

  Store.begin();
  data_init();

  Serial.println("Init ETC ...");
  ESC.attach(ESC_PIN,1000,2000); 

  for(uint8_t i=0;i<=100;i=i+10)
  {
    display.progressBar(i);
    delay(500);
    ESC.write(0);
  }

  // Initialise  timer
  timer = timerBegin(0, 80, true); // Timer 0, Prescaler 80, zähle im Up-Modus
  timerAttachInterrupt(timer, &ISR_onTimer, true); // Setze den Interrupt-Handler und aktiviere den Interrupt auf steigende Flanke
  timerAlarmWrite(timer, 1000000, true); // Setze den Alarm auf 1/4 Sekunde und aktiviere ihn
  timerAlarmEnable(timer); // Aktiviere den Alarm

  pinMode(39, INPUT_PULLDOWN);
  pinMode(HALL1_PIN, INPUT);
  pinMode(HALL2_PIN, INPUT);
  pinMode(HALL3_PIN, INPUT);
  pinMode(HALL4_PIN, INPUT);
  pinMode(HALL5_PIN, INPUT);
  pinMode(HALL6_PIN, INPUT);
  pinMode(HALL7_PIN, INPUT);
  pinMode(HALL8_PIN, INPUT);

  //Serial.println("Init light sensor interrupt ...");
  //attachInterrupt(LS1_PIN, ISR_LS1, RISING);

  Serial.println("Init hall sensor interrupts ...");
  attachInterrupt(digitalPinToInterrupt(HALL1_PIN), ISR_HALL1, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL2_PIN), ISR_HALL2, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL3_PIN), ISR_HALL3, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL4_PIN), ISR_HALL4, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL5_PIN), ISR_HALL5, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL6_PIN), ISR_HALL6, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL7_PIN), ISR_HALL7, FALLING);
  attachInterrupt(digitalPinToInterrupt(HALL8_PIN), ISR_HALL8, FALLING);  

  Serial.println(" Auf gehts ... ");
 
}

void loop() 
{

  capportal.handle();

  data_generate();

  count_secs(&run_time);
  speed_set();
  data_check();
  data_store();

  #ifdef HAS_MQTT
  mqtt.send_and_loop("infinity/data", json.create_json_string(AllData, ALL_DATA_COUNT, run_time));
  #endif
  
  display.show_values(speed, MIN_ESC_SPEED, MAX_ESC_SPEED, HallData, HALL_SENSORS_COUNT, run_time, capportal.localIP());

 // Serial.printf("PID ... (goal: %06.3f, current: %06.3f, err: %06.3f, integ: %06.3f, derive: %06.3f, output=%06.3f), esc_output=%06.3f \n", goal_speed, current_speed, err, integ, derivative, output, esc_output);

}

void count_secs(time_t* run_time)
{
  static time_t last_update_ms = 0;
  if(last_update_ms == 0)
  {
    last_update_ms = millis();
  }
  if(millis() - last_update_ms >= 1000)
  {
    last_update_ms = millis();
    *run_time = *run_time + 1;
  }
}

void speed_set()
{
  static uint8_t _count_meas = 0;
  static int _Drehregler = 0;
  static unsigned long _last_read_ms = 0;
  
  if( millis() - _last_read_ms > 25)
  {
    _last_read_ms = millis();
    int maxDrehregler = 0;
    int minDrehregler = 0;
    int sumDrehregler = 0;
    for(uint8_t i = 0; i<6;i++)
    {
      Drehregler = analogRead(POTI_PIN);   // Dieser Befehl liest den Wert des Potentiometers am analogen Pin A0 aus und speichert ihn unter der Variablen "Drehregler". Der Wert liegt zwischen 0 und 1023.
      if(i==0)
      {
         maxDrehregler = Drehregler;
         minDrehregler = Drehregler;
      }
      else
      {
        if(Drehregler < minDrehregler)
        {
          minDrehregler = Drehregler;
        }
        if(Drehregler > maxDrehregler)
        {
          maxDrehregler = Drehregler;
        }
      }
      sumDrehregler = sumDrehregler + Drehregler;
    }
    sumDrehregler = sumDrehregler - maxDrehregler - minDrehregler;
    sumDrehregler = sumDrehregler/4;
    #ifdef DUMMY_DATA
    speed = 1000;
    #else
    speed = map(sumDrehregler, 0, MAX_POTI_VALUE, MIN_SPEED, MAX_SPEED);   // Der "MAP-" Befehl wandelt den Messwert aus der Variablen "Drehregler" um, damit er am ESC verarbeitet werden kann. Der Zahlenbereich 0 bis 1023 wird dabei in einen Zahlenwert zwischen 0 und 180 umgewandelt.
    #endif
  }
}

void data_init()
{
  Serial.println("Init internal data ...");
  #ifdef MACHINE_RESTART
    run_time = 0;
    start_time = capportal.epochTime();
    Store.set_int(EEPROM_NAME_RUN_TIME , run_time );
    Store.set_int(EEPROM_NAME_START_EPOCHTIME, start_time );
    Serial.printf("Machine restarted. Set runtime to 0. Stored this value and the epoch start time %lu to EEPROM.\n", start_time);
  #else
    run_time = Store.get_int(EEPROM_NAME_RUN_TIME );
    start_time = Store.get_int(EEPROM_NAME_START_EPOCHTIME );
    Serial.printf("Machine runtime loaded from EPPROM: %lu seconds. Machine was started %s\n", run_time, capportal.localTime(start_time));
  #endif
  last_speed = 0;
  speed = 0;
  for (uint8_t i = 0; i < ALL_DATA_COUNT; i ++)
  {
    AllData[i].name = Data_names[i];
    AllData[i].unit = Data_units[i];
    AllData[i].value = 0;
    AllData[i].unit_factor = Data_units_factor[i];
    AllData[i].period_ticks = 0;
    #ifdef MACHINE_RESTART
    AllData[i].ticks = 0;
    #else
    AllData[i].ticks = Store.get_int(EEPROM_NAME_TICK+String(i));
    AllData[i].timestamp = Store.get_int(EEPROM_NAME_TIMESTAMP+String(i));    
    #endif
    AllData[i].last_update_ms = 0;
    AllData[i].timestamp = 0;
  }
  delay(500);
}

void data_check()
{
  unsigned long current_millis = millis();
  static unsigned long speedchanged_ms = current_millis ;
  /*
  Serial.printf("H1(pin %d): %d, H2(pin %d): %d, H3(pin %d): %d, H4(pin %d): %d, H5(pin %d) : %d, H6(pin %d) : %d, H7(pin %d) : %d, H8(pin %d) : %d.\n",
                HALL1_PIN, digitalRead(HALL1_PIN),
                HALL2_PIN, digitalRead(HALL2_PIN),
                HALL3_PIN, digitalRead(HALL3_PIN),
                HALL4_PIN, digitalRead(HALL4_PIN),
                HALL5_PIN, digitalRead(HALL5_PIN),
                HALL6_PIN, digitalRead(HALL6_PIN),
                HALL7_PIN, digitalRead(HALL7_PIN),
                HALL8_PIN, digitalRead(HALL8_PIN)                
                );
  
  */
  if(current_millis - speedchanged_ms >= UPDATE_TURN_VALUES_EVERY_MS)
  {


    HallData[0].value = (HallData[0].period_ticks * HallData[0].unit_factor);
    HallData[0].period_ticks = 0;
    Serial.printf("current_millis: %lu goal speed: %lu calc speed: %f (%lu ticks in %lu ms) \n", current_millis, speed, HallData[0].value, current_millis - speedchanged_ms);
    speedchanged_ms = current_millis;
  }


  for(uint8_t nr = 1; nr < HALL_SENSORS_COUNT; nr++)
  {
    HallData[nr].value = (HallData[nr].ticks * HallData[nr].unit_factor);
    HallData[nr].period_ticks = 0;
  }
}

void data_store()
{
  static unsigned long _last_stored_ms = 0;
  if(_last_stored_ms == 0)
  {
    _last_stored_ms = millis();
  }
  if(millis() - _last_stored_ms >= EEPROM_STORE_RUNTIME_ALL_MS)
  {
    _last_stored_ms = millis();
    Store.set_int(EEPROM_NAME_RUN_TIME, run_time);
    for(uint8_t nr = 0; nr < HALL_SENSORS_COUNT; nr++)
    {
      Store.set_int(EEPROM_NAME_TICK+String(nr), HallData[nr].ticks);
      Store.set_int(EEPROM_NAME_TIMESTAMP+String(nr), HallData[nr].timestamp);
    }
  }
}

void IRAM_ATTR ISR_HALL1()
{
  const uint8_t hallnr = 0;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL2()
{
  const uint8_t hallnr = 1;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL3()
{
  const uint8_t hallnr = 2;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL4()
{
  const uint8_t hallnr = 3;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL5()
{
  const uint8_t hallnr = 4;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL6()
{
  const uint8_t hallnr = 5;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL7()
{
  const uint8_t hallnr = 6;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}

void IRAM_ATTR ISR_HALL8()
{
  const uint8_t hallnr = 7;
  HallData[hallnr].ticks++;
  HallData[hallnr].period_ticks++;
  HallData[hallnr].timestamp = run_time;
  HallData[hallnr].last_update_ms = millis();
}




void IRAM_ATTR ISR_onTimer()
{
  current_speed = HallData[0].value;
  err = goal_speed - current_speed;
  integ = integ + err;
  derivative = (err - last_error);
  output = kp * err + ki * integ + kd * derivative;

  //Beschränkung der Ausgabe
  esc_output = constrain(output, MIN_ESC_SPEED, MAX_ESC_SPEED);
  //Serial.printf("PID ... (goal: %f, current: %06.3f, err: %06.3f, integ: %06.3f, derive: 06.3%f) output=%06.3f \n", goal_speed, current_speed, err, integ, derivative, output);

  //Aktualisierung der letzten Fehler- und Zeitwerte
  last_error = err;

  //Ansteuerung des Outputs
  
  ESC.write(esc_output); 
}

//this code is just for generating dummy data - in normal use not needed
void data_generate()
{
  #ifdef DUMMY_DATA

  static unsigned long _last_updated_ms = 0; 
  speed = 1000 + random(-250,150);
  unsigned long _wait_ms = map( speed, MIN_SPEED, MAX_SPEED, 60000/MIN_SPEED/4, 60000/MAX_SPEED/4);
  //Serial.printf("Speed is : %lu and _wait_ms is %lu\n", speed, _wait_ms);

  if(_last_updated_ms == 0)
  {
    _last_updated_ms = millis();
  }

  if(millis() - _last_updated_ms >= _wait_ms )
  {
    _last_updated_ms = millis();

    for(uint8_t i = 0; i < HALL_SENSORS_COUNT; i++)
    {
      if(i == 0)
      {
        HallData[i].ticks++;
        HallData[i].period_ticks++;
        HallData[i].last_update_ms = millis();
        HallData[i].timestamp = run_time;
      }
      else
      {
        //Serial.printf("i=%d HallData[i-1].ticks=%lu %4 = %d\n" , i, HallData[i-1].ticks , HallData[i-1].ticks % 4);
        if (HallData[i].last_update_ms != HallData[i-1].last_update_ms && HallData[i-1].ticks % 4 == 0)
        {
          HallData[i].ticks++;
          HallData[i].period_ticks++;
          HallData[i].last_update_ms = HallData[i-1].last_update_ms;
          HallData[i].timestamp = run_time;
        }
      }
    }
  }
  #endif
}
